@@ -1,4 +1,4 @@
-use std::convert::TryFrom;
+use std::{cmp::Ordering, convert::TryFrom};
 
 use base64::display::Base64Display;
 use chrono::{TimeZone, Utc};
@@ -6,7 +6,7 @@ use prost::{DecodeError, Message};
 use serde::ser::{Error, Serialize, SerializeMap, SerializeSeq, Serializer};
 
 use crate::{
-    descriptor::{Kind, MAP_ENTRY_VALUE_NUMBER},
+    descriptor::Kind,
     dynamic::{
         serde::{is_well_known_type, SerializeOptions},
         DynamicMessage, DynamicMessageField, MapKey, Value,
@@ -201,7 +201,7 @@ impl<'a> Serialize for SerializeWrapper<'a, ValueAndKind<'a>> {
             Value::Map(values) => {
                 let value_kind = match self.value.kind {
                     Kind::Message(message) if message.is_map_entry() => {
-                        message.get_field(MAP_ENTRY_VALUE_NUMBER).unwrap().kind()
+                        message.map_entry_value_field().kind()
                     }
                     _ => panic!(
                         "mismatch between DynamicMessage value {:?} and type {:?}",
@@ -358,12 +358,28 @@ fn serialize_duration<S>(
 where
     S: Serializer,
 {
-    let raw: prost_types::Duration = msg.transcode_to().map_err(decode_to_ser_err)?;
+    let mut raw: prost_types::Duration = msg.transcode_to().map_err(decode_to_ser_err)?;
 
-    if raw.nanos == 0 {
-        serializer.collect_str(&format_args!("{}s", raw.seconds))
-    } else {
-        serializer.collect_str(&format_args!("{}.{:0>9}s", raw.seconds, raw.nanos))
+    raw.normalize();
+    match (raw.seconds.cmp(&0), raw.nanos.cmp(&0)) {
+        (_, Ordering::Equal) => serializer.collect_str(&format_args!("{}s", raw.seconds)),
+        (Ordering::Less, Ordering::Greater) | (Ordering::Greater, Ordering::Less) => {
+            Err(Error::custom("inconsistent signs for duration"))
+        }
+        (Ordering::Equal | Ordering::Less, Ordering::Less) => {
+            serializer.collect_str(&format_args!(
+                "-{}.{:0>9}s",
+                raw.seconds.unsigned_abs(),
+                raw.nanos.unsigned_abs()
+            ))
+        }
+        (Ordering::Equal | Ordering::Greater, Ordering::Greater) => {
+            serializer.collect_str(&format_args!(
+                "{}.{:0>9}s",
+                raw.seconds.unsigned_abs(),
+                raw.nanos.unsigned_abs()
+            ))
+        }
     }
 }
 
